@@ -1,8 +1,6 @@
 #!/home/ben/software/install/bin/perl
 use warnings;
 use strict;
-use utf8;
-use FindBin '$Bin';
 use Gzip::Faster;
 my $gf = Gzip::Faster->new ();
 $gf->file_name ("blash.gz");
